#pragma once
#include "xapo.h"
#include <assert.h>
#define _CRTDBG_MAP_ALLOC
#include <crtdbg.h>
#define new new(_NORMAL_BLOCK, __FILE__, __LINE__)
typedef float FLOAT32;
class  __declspec(novtable)  XAPOBase :
	public IXAPO
{
public:
	XAPOBase(const XAPO_REGISTRATION_PROPERTIES* pRegistrationProperties):refCnt(0){
		this->m_pRegistrationProperties=pRegistrationProperties;
	}
	virtual ~XAPOBase(void);
	STDMETHOD(QueryInterface) (REFIID riid, __deref_out_opt void** ppInterface)
    {
        assert(ppInterface != NULL);
        HRESULT hr = S_OK;
        if (riid == __uuidof(IXAPO)) {
            *ppInterface = static_cast<IXAPO*>(this);
            XAPOBase::AddRef();
			return hr;
        }
		return E_NOINTERFACE;
    }
    STDMETHOD_(ULONG, AddRef)() { refCnt++;return refCnt;}
    STDMETHOD_(ULONG, Release)() {
		refCnt--;
		if(refCnt==0){
			//EffSynthō폜邽
			//delete this;
			return 0;
		}
		return refCnt;
	}
	STDMETHOD(GetRegistrationProperties) (THIS_ __deref_out XAPO_REGISTRATION_PROPERTIES** ppRegistrationProperties) ;
	STDMETHOD(IsInputFormatSupported) (THIS_ const WAVEFORMATEX* pOutputFormat, const WAVEFORMATEX* pRequestedInputFormat, __deref_opt_out WAVEFORMATEX** ppSupportedInputFormat) ;
	STDMETHOD(IsOutputFormatSupported) (THIS_ const WAVEFORMATEX* pInputFormat, const WAVEFORMATEX* pRequestedOutputFormat, __deref_opt_out WAVEFORMATEX** ppSupportedOutputFormat) ;
	STDMETHOD(Initialize) (THIS_ __in_bcount_opt(DataByteSize) const void* pData, UINT32 DataByteSize) ;
	STDMETHOD_(void, Reset) (THIS) ;
	STDMETHOD(LockForProcess) (THIS_ UINT32 InputLockedParameterCount, __in_ecount_opt(InputLockedParameterCount) const XAPO_LOCKFORPROCESS_BUFFER_PARAMETERS* pInputLockedParameters, UINT32 OutputLockedParameterCount, __in_ecount_opt(OutputLockedParameterCount) const XAPO_LOCKFORPROCESS_BUFFER_PARAMETERS* pOutputLockedParameters) ;
	BOOL IsLocked(){
		return m_fIsLocked;
	}
	STDMETHOD_(void, UnlockForProcess) (THIS) ;
	STDMETHOD_(UINT32, CalcInputFrames) (THIS_ UINT32 OutputFrameCount) ;
	STDMETHOD_(UINT32, CalcOutputFrames) (THIS_ UINT32 InputFrameCount) ;
private:
	const XAPO_REGISTRATION_PROPERTIES* m_pRegistrationProperties; // pointer to registration properties of the XAPO, set via constructor

    void*    m_pfnMatrixMixFunction;    // optimal matrix function pointer, used for thru processing
    FLOAT32* m_pfl32MatrixCoefficients; // matrix coefficient table, used for thru processing
    UINT32   m_nSrcFormatType;          // input format type, used for thru processing
    BOOL     m_fIsScalarMatrix;         // TRUE if m_pfl32MatrixCoefficients is diagonal matrix with all main diagonal entries equal, i.e. m_pfnMatrixMixFunction only used for type conversion (no channel conversion), used for thru processing
    BOOL     m_fIsLocked;               // TRUE if XAPO locked via CXAPOBase.LockForProcess

	ULONG refCnt;
};

class __declspec(novtable) XAPOParametersBase: public XAPOBase, public IXAPOParameters {
private:
    BYTE*  m_pParameterBlocks;           // three contiguous process parameter blocks used for synchronization, user responsible for initialization of parameter blocks before IXAPO::Process/SetParameters/GetParameters called
    BYTE*  m_pCurrentParameters;         // pointer to current process parameters, must be aligned for atomic operations
    BYTE*  m_pCurrentParametersInternal; // pointer to current process parameters (temp pointer read by SetParameters/BeginProcess/EndProcess)
    UINT32 m_uCurrentParametersIndex;    // index of current process parameters
    UINT32 m_uParameterBlockByteSize;    // size of a single parameter block in bytes, must be > 0
    BOOL   m_fNewerResultsReady;         // TRUE if there exists new processing results not yet picked up by GetParameters(), must be aligned for atomic operations
    BOOL   m_fProducer;                  // IXAPO::Process produces data to be returned by GetParameters(); SetParameters() disallowed
public:
    XAPOParametersBase (const XAPO_REGISTRATION_PROPERTIES* pRegistrationProperties, BYTE* pParameterBlocks, UINT32 uParameterBlockByteSize, BOOL fProducer);
    virtual ~XAPOParametersBase ();

    // IUnknown methods:
    // retrieves the requested interface pointer if supported
    STDMETHOD(QueryInterface) (REFIID riid, __deref_out_opt void** ppInterface)
    {
		assert(_CrtCheckMemory());
        assert(ppInterface != NULL);
        HRESULT hr = S_OK;

        if (riid == __uuidof(IXAPOParameters)) {
            *ppInterface = static_cast<IXAPOParameters*>(this);
            XAPOBase::AddRef();
        } else {
            hr = XAPOBase::QueryInterface(riid, ppInterface);
        }

        return hr;
    }
    STDMETHOD_(ULONG, AddRef)() { return XAPOBase::AddRef(); }
    STDMETHOD_(ULONG, Release)() { return XAPOBase::Release(); }
    STDMETHOD_(void, SetParameters) (__in_bcount(ParameterByteSize) const void* pParameters, UINT32 ParameterByteSize);
    STDMETHOD_(void, GetParameters) (__out_bcount(ParameterByteSize) void* pParameters, UINT32 ParameterByteSize);
    virtual void OnSetParameters (const void*, UINT32) { }
    BOOL ParametersChanged ();
    BYTE* BeginProcess ();
    void EndProcess ();
};